@@ -16,13 +16,14 @@ namespace charls {
 namespace impl {
 
 #else
+#include <stddef.h>
 #include <stdint.h>
 #endif
 
 // The following enum values are for C applications, for C++ the enums are defined after these definitions.
 // For the documentation, see the C++ enums definitions.
 
-RETURN_TYPE_SUCCESS_(return == 0)
+CHARLS_RETURN_TYPE_SUCCESS(return == 0)
 enum charls_jpegls_errc
 {
     CHARLS_JPEGLS_ERRC_SUCCESS = 0,
@@ -50,6 +51,10 @@ enum charls_jpegls_errc
     CHARLS_JPEGLS_ERRC_INVALID_JPEGLS_PRESET_PARAMETER_TYPE = 22,
     CHARLS_JPEGLS_ERRC_JPEGLS_PRESET_EXTENDED_PARAMETER_TYPE_NOT_SUPPORTED = 23,
     CHARLS_JPEGLS_ERRC_MISSING_END_OF_SPIFF_DIRECTORY = 24,
+    CHARLS_JPEGLS_ERRC_UNEXPECTED_RESTART_MARKER = 25,
+    CHARLS_JPEGLS_ERRC_RESTART_MARKER_NOT_FOUND = 26,
+    CHARLS_JPEGLS_ERRC_CALLBACK_FAILED = 27,
+    CHARLS_JPEGLS_ERRC_END_OF_IMAGE_MARKER_NOT_FOUND = 28,
     CHARLS_JPEGLS_ERRC_INVALID_ARGUMENT_WIDTH = 100,
     CHARLS_JPEGLS_ERRC_INVALID_ARGUMENT_HEIGHT = 101,
     CHARLS_JPEGLS_ERRC_INVALID_ARGUMENT_COMPONENT_COUNT = 102,
@@ -57,15 +62,17 @@ enum charls_jpegls_errc
     CHARLS_JPEGLS_ERRC_INVALID_ARGUMENT_INTERLEAVE_MODE = 104,
     CHARLS_JPEGLS_ERRC_INVALID_ARGUMENT_NEAR_LOSSLESS = 105,
     CHARLS_JPEGLS_ERRC_INVALID_ARGUMENT_JPEGLS_PC_PARAMETERS = 106,
-    CHARLS_JPEGLS_ERRC_INVALID_ARGUMENT_SPIFF_ENTRY_SIZE = 110,
+    CHARLS_JPEGLS_ERRC_INVALID_ARGUMENT_SIZE = 110,
     CHARLS_JPEGLS_ERRC_INVALID_ARGUMENT_COLOR_TRANSFORMATION = 111,
+    CHARLS_JPEGLS_ERRC_INVALID_ARGUMENT_STRIDE = 112,
+    CHARLS_JPEGLS_ERRC_INVALID_ARGUMENT_ENCODING_OPTIONS = 113,
     CHARLS_JPEGLS_ERRC_INVALID_PARAMETER_WIDTH = 200,
     CHARLS_JPEGLS_ERRC_INVALID_PARAMETER_HEIGHT = 201,
     CHARLS_JPEGLS_ERRC_INVALID_PARAMETER_COMPONENT_COUNT = 202,
     CHARLS_JPEGLS_ERRC_INVALID_PARAMETER_BITS_PER_SAMPLE = 203,
     CHARLS_JPEGLS_ERRC_INVALID_PARAMETER_INTERLEAVE_MODE = 204,
     CHARLS_JPEGLS_ERRC_INVALID_PARAMETER_NEAR_LOSSLESS = 205,
-    CHARLS_JPEGLS_ERRC_INVALID_PARAMETER_JPEGLS_PC_PARAMETERS = 206
+    CHARLS_JPEGLS_ERRC_INVALID_PARAMETER_JPEGLS_PRESET_PARAMETERS = 206
 };
 
 enum charls_interleave_mode
@@ -75,6 +82,14 @@ enum charls_interleave_mode
     CHARLS_INTERLEAVE_MODE_SAMPLE = 2
 };
 
+enum charls_encoding_options
+{
+    CHARLS_ENCODING_OPTIONS_NONE = 0,
+    CHARLS_ENCODING_OPTIONS_EVEN_DESTINATION_SIZE = 1,
+    CHARLS_ENCODING_OPTIONS_INCLUDE_VERSION_NUMBER = 2,
+    CHARLS_ENCODING_OPTIONS_INCLUDE_PC_PARAMETERS_JAI = 4
+};
+
 enum charls_color_transformation
 {
     CHARLS_COLOR_TRANSFORMATION_NONE = 0,
@@ -155,8 +170,8 @@ namespace charls {
 /// <summary>
 /// Defines the result values that are returned by the CharLS API functions.
 /// </summary>
-RETURN_TYPE_SUCCESS_(return == 0)
-enum class jpegls_errc
+CHARLS_RETURN_TYPE_SUCCESS(return == 0)
+enum class CHARLS_NO_DISCARD jpegls_errc
 {
     /// <summary>
     /// The operation completed without errors.
@@ -285,6 +300,27 @@ enum class jpegls_errc
     /// </summary>
     missing_end_of_spiff_directory = impl::CHARLS_JPEGLS_ERRC_MISSING_END_OF_SPIFF_DIRECTORY,
 
+    /// <summary>
+    /// This error is returned when a restart marker is found outside the encoded entropy data.
+    /// </summary>
+    unexpected_restart_marker = impl::CHARLS_JPEGLS_ERRC_UNEXPECTED_RESTART_MARKER,
+
+    /// <summary>
+    /// This error is returned when an expected restart marker is not found. It may indicate data corruption in the JPEG-LS
+    /// byte stream.
+    /// </summary>
+    restart_marker_not_found = impl::CHARLS_JPEGLS_ERRC_RESTART_MARKER_NOT_FOUND,
+
+    /// <summary>
+    /// This error is returned when a callback function returns a non zero value.
+    /// </summary>
+    callback_failed = impl::CHARLS_JPEGLS_ERRC_CALLBACK_FAILED,
+
+    /// <summary>
+    /// This error is returned when the End of Image (EOI) marker could not be found.
+    /// </summary>
+    end_of_image_marker_not_found = impl::CHARLS_JPEGLS_ERRC_END_OF_IMAGE_MARKER_NOT_FOUND,
+
     /// <summary>
     /// The argument for the width parameter is outside the range [1, 65535].
     /// </summary>
@@ -322,15 +358,25 @@ enum class jpegls_errc
     invalid_argument_jpegls_pc_parameters = impl::CHARLS_JPEGLS_ERRC_INVALID_ARGUMENT_JPEGLS_PC_PARAMETERS,
 
     /// <summary>
-    /// The argument for the entry size parameter is outside the range [0, 65528].
+    /// The argument for the size parameter is outside the valid range.
     /// </summary>
-    invalid_argument_spiff_entry_size = impl::CHARLS_JPEGLS_ERRC_INVALID_ARGUMENT_SPIFF_ENTRY_SIZE,
+    invalid_argument_size = impl::CHARLS_JPEGLS_ERRC_INVALID_ARGUMENT_SIZE,
 
     /// <summary>
     /// The argument for the color component is not (None, Hp1, Hp2, Hp3) or invalid in combination with component count.
     /// </summary>
     invalid_argument_color_transformation = impl::CHARLS_JPEGLS_ERRC_INVALID_ARGUMENT_COLOR_TRANSFORMATION,
 
+    /// <summary>
+    /// The stride argument does not match with the frame info and buffer size.
+    /// </summary>
+    invalid_argument_stride = impl::CHARLS_JPEGLS_ERRC_INVALID_ARGUMENT_STRIDE,
+
+    /// <summary>
+    /// The encoding options argument has an invalid value.
+    /// </summary>
+    invalid_argument_encoding_options = impl::CHARLS_JPEGLS_ERRC_INVALID_ARGUMENT_ENCODING_OPTIONS,
+
     /// <summary>
     /// This error is returned when the stream contains a width parameter defined more then once or in an incompatible way.
     /// </summary>
@@ -342,7 +388,7 @@ enum class jpegls_errc
     invalid_parameter_height = impl::CHARLS_JPEGLS_ERRC_INVALID_PARAMETER_HEIGHT,
 
     /// <summary>
-    /// This error is returned when the stream contains a component count parameter outside the range [1,255]
+    /// This error is returned when the stream contains a component count parameter outside the range [1,255] for SOF or [1,4] for SOS.
     /// </summary>
     invalid_parameter_component_count = impl::CHARLS_JPEGLS_ERRC_INVALID_PARAMETER_COMPONENT_COUNT,
 
@@ -364,9 +410,9 @@ enum class jpegls_errc
     invalid_parameter_near_lossless = impl::CHARLS_JPEGLS_ERRC_INVALID_PARAMETER_NEAR_LOSSLESS,
 
     /// <summary>
-    /// This error is returned when the stream contains an invalid JPEG-LS preset coding parameters segment.
+    /// This error is returned when the stream contains an invalid JPEG-LS preset parameters segment.
     /// </summary>
-    invalid_parameter_jpegls_pc_parameters = impl::CHARLS_JPEGLS_ERRC_INVALID_PARAMETER_JPEGLS_PC_PARAMETERS,
+    invalid_parameter_jpegls_preset_parameters = impl::CHARLS_JPEGLS_ERRC_INVALID_PARAMETER_JPEGLS_PRESET_PARAMETERS,
 
     // Legacy enumerator names, will be removed in next major release. Not tagged with [[deprecated]] as that is a C++17
     // extension.
@@ -412,6 +458,50 @@ enum class interleave_mode
     Sample = sample
 };
 
+/// <summary>
+/// Defines options that can be enabled during the encoding process.
+/// These options can be combined.
+/// </summary>
+enum class encoding_options
+{
+    /// <summary>
+    /// No special encoding option is defined.
+    /// </summary>
+    none = impl::CHARLS_ENCODING_OPTIONS_NONE,
+
+    /// <summary>
+    /// Ensures that the generated encoded data has an even size by adding
+    /// an extra 0xFF byte to the End Of Image (EOI) marker.
+    /// DICOM requires that data is always even. This can be done by adding a zero padding byte
+    /// after the encoded data or with this option.
+    /// This option is not default enabled.
+    /// </summary>
+    even_destination_size = impl::CHARLS_ENCODING_OPTIONS_EVEN_DESTINATION_SIZE,
+
+    /// <summary>
+    /// Add a comment (COM) segment with the content: "charls [version-number]" to the encoded data.
+    /// Storing the used encoder version can be helpful for long term archival of images.
+    /// This option is not default enabled.
+    /// </summary>
+    include_version_number = impl::CHARLS_ENCODING_OPTIONS_INCLUDE_VERSION_NUMBER,
+
+    /// <summary>
+    /// Writes explicitly the default JPEG-LS preset coding parameters when the
+    /// bits per sample is larger then 12 bits.
+    /// The Java Advanced Imaging (JAI) JPEG-LS codec has a defect that causes it to use invalid
+    /// preset coding parameters for these types of images.
+    /// Most users of this codec are aware of this problem and have implemented a work-around.
+    /// This option is default enabled. Will not be default enabled in the next major version upgrade.
+    /// </summary>
+    include_pc_parameters_jai = impl::CHARLS_ENCODING_OPTIONS_INCLUDE_PC_PARAMETERS_JAI
+};
+
+constexpr encoding_options operator|(const encoding_options lhs, const encoding_options rhs) noexcept
+{
+    using T = std::underlying_type_t<encoding_options>;
+    return static_cast<encoding_options>(static_cast<T>(lhs) | static_cast<T>(rhs));
+}
+
 /// <summary>
 /// Defines color space transformations as defined and implemented by the JPEG-LS library of HP Labs.
 /// These color space transformation decrease the correlation between the 3 color components, resulting in better encoding
@@ -717,17 +807,14 @@ using ColorTransformation CHARLS_DEPRECATED = color_transformation;
 
 } // namespace charls
 
-namespace std {
-
 template<>
-struct is_error_code_enum<charls::jpegls_errc> final : true_type
+struct std::is_error_code_enum<charls::jpegls_errc> final : std::true_type
 {
 };
 
-} // namespace std
-
 using charls_jpegls_errc = charls::jpegls_errc;
 using charls_interleave_mode = charls::interleave_mode;
+using charls_encoding_options = charls::encoding_options;
 using charls_color_transformation = charls::color_transformation;
 
 using charls_spiff_profile_id = charls::spiff_profile_id;
@@ -749,6 +836,7 @@ constexpr std::size_t ErrorMessageSize = 256;
 
 typedef enum charls_jpegls_errc charls_jpegls_errc;
 typedef enum charls_interleave_mode charls_interleave_mode;
+typedef enum charls_encoding_options charls_encoding_options;
 typedef enum charls_color_transformation charls_color_transformation;
 
 typedef int32_t charls_spiff_profile_id;
@@ -1014,14 +1102,25 @@ struct JlsParameters
     struct JfifParameters jfif;
 };
 
-
 #ifdef __cplusplus
 
+/// <summary>
+/// Function definition for a callback handler that will be called when a comment (COM) segment is found.
+/// </summary>
+/// <remarks>
+/// </remarks>
+/// <param name="data">Reference to the data of the COM segment.</param>
+/// <param name="size">Size in bytes of the data of the COM segment.</param>
+/// <param name="user_context">Free to use context information that can be set during the installation of the
+/// handler.</param>
+using charls_at_comment_handler = int32_t(CHARLS_API_CALLING_CONVENTION*)(const void* data, size_t size, void* user_context);
+
 namespace charls {
 
 using spiff_header = charls_spiff_header;
 using frame_info = charls_frame_info;
 using jpegls_pc_parameters = charls_jpegls_pc_parameters;
+using at_comment_handler = charls_at_comment_handler;
 
 static_assert(sizeof(spiff_header) == 40, "size of struct is incorrect, check padding settings");
 static_assert(sizeof(frame_info) == 16, "size of struct is incorrect, check padding settings");
@@ -1031,8 +1130,13 @@ static_assert(sizeof(jpegls_pc_parameters) == 20, "size of struct is incorrect,
 
 #else
 
+typedef void(CHARLS_API_CALLING_CONVENTION* charls_at_comment_handler)(const void* data, size_t size, void* user_context);
+
 typedef struct charls_spiff_header charls_spiff_header;
 typedef struct charls_frame_info charls_frame_info;
 typedef struct charls_jpegls_pc_parameters charls_jpegls_pc_parameters;
 
+typedef struct JlsParameters JlsParameters;
+typedef struct JlsRect JlsRect;
+
 #endif
